/***************************************************************
 * Proyecto CoMVeT                                             *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso    *
 * Escuela Tcnica Superior de Ingeniera Informtica          *
 * Universidad de Mlaga - 2010                                *
 *                                                             *
 * Actuador.cpp                                                *
 * Gestiona la salida de las seales de control por el puerto  *
 * paralelo (implementacin)                                   *
 *                                                             *
 ***************************************************************/
//---------------------------------------------------------------------------

#pragma hdrstop

#include "Actuador.h"
//---------------------------------------------------------------------------

#pragma package(smart_init)

TActuador::TActuador()
{
   cadenaError = "";
   error = false;
   habilitado = false;

	/* Carga la dll */
	UnicodeString dll = "inpout32.dll";
	hLib = LoadLibrary(dll.w_str());

   if (hLib == NULL) {
      error = true;
      cadenaError = "Error cargando dll";
   } else {
     /* Puntero a la funcin dentro de la dll */
      oup32fp = (oupfuncPtr) GetProcAddress(hLib, "Out32");

      if (oup32fp == NULL) {
         error = true;
         cadenaError = "Error obteniendo el puntero a la funcin de salida";
      }
   }
}

TActuador::~TActuador()
{
    FreeLibrary(hLib);
}

bool TActuador::movimiento(AC_Accion_t accion, GI_Giro_t giro)
{
   if (habilitado) {
      short valor;
      // Accin
      switch (accion) {
         case AC_Ninguna:   valor = PUERTO_NINGUNA; break;
         case AC_Centro:    valor = PUERTO_CENTRO; break;
         case AC_Adelante:  valor = PUERTO_ARRIBA; break;
         case AC_Atras:     valor = PUERTO_ABAJO; break;
      default:
         valor = PUERTO_NINGUNA;
      }
      // Giro
      switch (giro) {
         case GI_Ninguno:   break;
         case GI_Izquierda: valor = valor | PUERTO_IZQUIERDA; break;
         case GI_Derecha:   valor = valor | PUERTO_DERECHA; break;
      default:
         ;
      }
      // Vuelca y devuelve el resultado
      return puerto(valor);
   } else return false; // si no se encuentra habilitado devuelve false
}

bool TActuador::habilitar(bool activo)
{
   // Slo se habilita si no hay error
   habilitado = activo && !error;
   // Inicializa el puerto
   inicializar();
   return habilitado;
}

bool TActuador::inicializar()
{
   return puerto(PUERTO_NINGUNA);
}

bool TActuador::estadoError()
{
   return error;
}

UnicodeString TActuador::textoError()
{
   return cadenaError;
}

bool TActuador::puerto(const short valor)
{
   // Vuelca el valor en el puerto utilizando la dll
   (oup32fp)(PUERTO_PARALELO,valor);
   return true;
}


