/***************************************************************
 * Proyecto CoMVeT                                             *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso    *
 * Escuela Tcnica Superior de Ingeniera Informtica          *
 * Universidad de Mlaga - 2010                                *
 *                                                             *
 * Actuador.h                                                  *
 * Gestiona la salida de las seales de control por el puerto  *
 * paralelo (cabecera)                                         *
 *                                                             *
 ***************************************************************/
//---------------------------------------------------------------------------

#ifndef ActuadorH
#define ActuadorH
#include "Tipos.h"
#include <system.hpp>

//---------------------------------------------------------------------------
/* Valores a volcar en el puerto paralelo
   para cada una de las acciones posibles */
const char PUERTO_NINGUNA   = 0;
const char PUERTO_CENTRO    = 0;
const char PUERTO_ARRIBA    = 1;
const char PUERTO_ABAJO     = 2;
const char PUERTO_IZQUIERDA = 4;
const char PUERTO_DERECHA   = 8;
/* Puerto de salida */
const short PUERTO_PARALELO = 0x378;
/* funcin de salida en dll */
typedef void (_stdcall *oupfuncPtr)(short portaddr, short datum);

//---------------------------------------------------------------------------
class TActuador
{
private:
   HINSTANCE hLib;            // Instancia para la dll que accede al puerto
   oupfuncPtr oup32fp;        // Puntero a la funcin de salida
   bool habilitado;           // Almacena el estado del actuador
   bool error;                // Estado de inicializacin errneo
   UnicodeString cadenaError; // Cadena con el ltimo error producido

public:
   /* Constructor */
   TActuador();

   /* Destructor */
   ~TActuador();

   /* movimiento(AC_Accion_t accion, GI_Giro_t giro)
      Compone la accin y el giro pasados como parmetro y los vuelca en el puerto paralelo
      Devuelve true si no se han producido errores */
   bool movimiento(AC_Accion_t accion, GI_Giro_t giro);

   /* habilitar(bool activo)
      habilita / inhabilita el volcado de los datos en el puerto paralelo
      devuelve el nuevo estado del actuador */
   bool habilitar(bool activo);

   /* inicializar()
      Inicializa el puerto volcando en las lneas el valor PUERTO_NINGUNA */
   bool inicializar();

   /* estadoError()
      Devuelve true si se ha producido un error */
   bool estadoError();

   /* textoError()
      Devuelve la descripcin del ltimo error ocurrido */
   UnicodeString textoError();

private:
   /* puerto(const short valor)
      Vuelca en el puerto paralelo el valor pasado como parmetro
      Devuelve true si no se han producido errores */
   bool puerto(const short valor);

};
#endif


