/***************************************************************
 * Proyecto CoMVeT                                             *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso    *
 * Escuela Tcnica Superior de Ingeniera Informtica          *
 * Universidad de Mlaga - 2010                                *
 *                                                             *
 * Grafica.cpp                                                 *
 * Clase que gestiona la grfica de seales (implementacin)   *
 *                                                             *
 ***************************************************************/
//---------------------------------------------------------------------------

#pragma hdrstop

#include "Grafica.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

TGrafica::TGrafica(TPaintBox *pb, TImage *im)
{
   // Iniciliza valores
   this->pb = pb;
   this->pb->OnPaint = pintar;
   this->im = im;
   // Inicialmente inhabilitada
   estado(false);
   // Limpia el paintbox
   limpiar();
}

void TGrafica::estado(bool habilitada)
{
   // Cambia el estado segn el parmetro
   this->habilitada = habilitada;
   im->Visible = habilitada;
}

void TGrafica::limpiar()
{
   // Iniciliza los valores a 0
   unsigned short i,j;
   x = 0;
   for (i = 0; i < NUM_GRAFICAS; i++) {
      for (j = 0; j < NUM_PUNTOS; j++) {
         y[i][j] = 0;
      }  
   }
}

void TGrafica::anadir(unsigned short* valores)
{
   unsigned short i,j;

   x++;
   if (x >= NUM_PUNTOS) {
      // Desplaza si se llega al final
      for (i = 0; i < NUM_GRAFICAS; i++) {
         for (j = DESPLAZAMIENTO; j < NUM_PUNTOS; j++) {
            y[i][j-DESPLAZAMIENTO] = y[i][j];
         }
         for (j = NUM_PUNTOS-DESPLAZAMIENTO; j < NUM_PUNTOS; j++) {
            y[i][j] = 0;
         }
      }
      x = NUM_PUNTOS-DESPLAZAMIENTO;
   }

   // Aade nuevos valores
   for (i = 0; i < NUM_GRAFICAS; i++) {
      y[i][x] = valores[i];
   }
   // Repinta para reflejar los cambios
   pb->Repaint();
}

void __fastcall TGrafica::pintar(TObject *Sender)
{
   TCanvas *canvas = pb->Canvas;
   if (habilitada) {
      unsigned short senal;
      int xx;
      // Recuadro grfica
      canvas->Brush->Color = clWindow;
      canvas->Pen->Color = clBlack;
      canvas->Rectangle(0,0,pb->Width,pb->Height);
      // Nivel de referencia de cada grfica
      canvas->Pen->Color = clSilver;
      for (senal = 0; senal < NUM_GRAFICAS; senal++) {
         canvas->MoveTo(0,y0[senal]);
         canvas->LineTo(NUM_PUNTOS,y0[senal]);
      }
      // Puntos iniciales y grfica
      for (senal = 0; senal < NUM_GRAFICAS; senal++) {
         canvas->Pen->Color = color[senal];
         canvas->MoveTo(0,y0[senal]);
         for (xx = 0; xx < x; xx++) {
            canvas->LineTo(xx,y0[senal]-y[senal][xx]);
         }
      }
   } else {
      // Si no se encuentra habilitada se pinta de gris
      canvas->Brush->Color = clBtnFace;
      canvas->Pen->Color = clDkGray;
      canvas->Rectangle(0,0,pb->Width,pb->Height);
   }

}


