/*********************************************************************
 * Proyecto CoMVeT                                                   *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso          *
 * Escuela Tcnica Superior de Ingeniera Informtica                *
 * Universidad de Mlaga - 2010                                      *
 *                                                                   *
 * Main.cpp                                                          *
 * Interfaz grfica y punto de entrada del programa (implementacin) *
 *                                                                   *
 *********************************************************************/
// ---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Main.h"
#include "ConexionCasco.h"
#include "ConexionEmulador.h"
#include "about.h"
#include "IniFiles.hpp";
#include "WinHelpViewer.hpp"
#include <ustring.h>
#include <math.h>
// ---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "cgauges"
#pragma link "WinHelpViewer"
#pragma resource "*.dfm"

TFMain *FMain;
// ---------------------------------------------------------------------------

__fastcall TFMain::TFMain(TComponent* Owner) : TForm(Owner) {
	// Informacin de la versin desde el ejecutable
	wchar_t* nombre = Application->ExeName.w_str();
	wchar_t* buffer;
	unsigned long longitud, handle;
   unsigned int bufLen;
	longitud = GetFileVersionInfoSize(nombre, &handle);
   if (longitud) {
      LPTSTR datos = (LPTSTR) malloc(longitud);
      if (datos) {
         if (GetFileVersionInfo(nombre, handle, longitud, datos)) {
            if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\ProductVersion"), (LPVOID *) &buffer, &bufLen)) {
               VERSION = UnicodeString(buffer);
            }
            if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\FileVersion"), (LPVOID *) &buffer, &bufLen)) {
               BUILD = UnicodeString(buffer);
            }
            if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\Comments"), (LPVOID *) &buffer, &bufLen)) {
               FECHA_VERSION = UnicodeString(buffer);
            }
            if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\InternalName"), (LPVOID *) &buffer, &bufLen)) {
               NOMBRE = UnicodeString(buffer);
            }
				if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\ProductName"), (LPVOID *) &buffer, &bufLen)) {
               PROYECTO = UnicodeString(buffer);
            }
            if (VerQueryValue(datos, TEXT("\\StringFileInfo\\0C0A04E4\\LegalCopyright"), (LPVOID *) &buffer, &bufLen)) {
               AUTOR = UnicodeString(buffer);
            }
         }
         free(datos);
      }
   }

   // Lectura de la configuracin guardada
   ficheroINI = ExtractFilePath(Application->ExeName)+
                StringReplace(ExtractFileName(Application->ExeName),".exe",".ini",TReplaceFlags() << rfIgnoreCase);
   TIniFile *F;
   try {
      try {
         F = new TIniFile(ficheroINI);
         TrackBarUmbralAvRet->Position = F->ReadInteger("UMBRALES","AVRET",5);
         TrackBarUmbralGiro->Position = F->ReadInteger("UMBRALES","GIRO",5);
      } catch (const Exception& e) {
         TrackBarUmbralAvRet->Position = 5;
         TrackBarUmbralGiro->Position = 5;
      }
   } __finally {
      delete F;
   }
   TrackBarUmbralAvRetChange(TrackBarUmbralAvRet);
   TrackBarUmbralGiroChange(TrackBarUmbralGiro);

   // Fichero de ayuda
   Application->HelpFile = ExtractFilePath(Application->ExeName)+
                           StringReplace(ExtractFileName(Application->ExeName),".exe",".hlp",TReplaceFlags() << rfIgnoreCase);

   // Inicializacin de los atributos
   conectado = false;
   continuar = false;
   manual = true;
   perfilCargado = false;
   ficheroPerfil = "";
   inicializarGiro = false;
   giroManual = GI_Ninguno;
   grafica = new TGrafica(PaintBoxGrafica,ImageGrafica);

   // Inicializacin de los componentes del formulario
   RichEditRegistro->Clear();
   EditIp1->Text = "127";
   EditIp2->Text = "0";
   EditIp3->Text = "0";
   EditIp4->Text = "1";
   EditPuerto->Text = PUERTO_EMULADOR;

   PanelCalibrandoAcelerometros->Visible = false;

   ActionManual->Checked = true;
   ActionActuador->Checked = false;
   LabelFicheroPerfil->Caption = "";
   LabelFicheroPerfil->Hint = "";
   // Configura el formulario
   formulario();
   // Informacin de versin
   registro(RG_Mensaje,NOMBRE + " V:" + VERSION + " (" + BUILD + ") - " + FECHA_VERSION);
   registro(RG_Mensaje,"2010 - " + AUTOR);

   // Inicializacin del actuador
   actuador = new TActuador();
   if (actuador->estadoError()) {
      registro(RG_Error,actuador->textoError());
   } else {
      actuador->inicializar();
      actuador->habilitar(ActionActuador->Checked);
   }
}
// ---------------------------------------------------------------------------

void TFMain::registro(RG_Registro_t tipo, UnicodeString cadena) {
//	registro(tipo, cadena.w_str());
	int ini = RichEditRegistro->Lines->Text.Length() - RichEditRegistro->Lines->Count;
	RichEditRegistro->Lines->Append(cadena);
	RichEditRegistro->SelStart = ini;
	RichEditRegistro->SelLength = cadena.Length(); //StrLen(cadena);
	RichEditRegistro->SelAttributes->Color = COLORES_REGISTRO[tipo];

	// Desplaza al final
	int i;
	TWMVScroll scroll;
	scroll.Msg = WM_VSCROLL;
	for (i=0; i<=RichEditRegistro->Lines->Count; i++) {
		scroll.ScrollCode = SB_LINEDOWN;
		scroll.Pos = 0;
		RichEditRegistro->Dispatch(&scroll);
	}

	RichEditRegistro->SelLength = 0;
	Application->ProcessMessages();
}
// ---------------------------------------------------------------------------

void TFMain::imagen(AC_Accion_t ac, GI_Giro_t gi, bool activoAB, bool activoID) {
   // Calcula los ndices
   unsigned short indiceAB = ac;
   if ((activoAB) && (indiceAB != 0)) indiceAB = indiceAB + 3;
   unsigned short indiceID = gi;
   if ((activoID) && (indiceID != 0)) indiceID = indiceID + 2;

   // Actualiza slo si ha cambiado (evita parpadeo)
   if (indiceAB != imgFlechasAB) {
      imgFlechasAB = indiceAB;
      ImageListFlechasArribaAbajo->GetBitmap(imgFlechasAB, ImageFlechasArribaAbajo->Picture->Bitmap);
      ImageFlechasArribaAbajo->Refresh();
   }
   if (indiceID != imgFlechasID) {
      imgFlechasID = indiceID;
      ImageListFlechasIzquierdaDerecha->GetBitmap(imgFlechasID, ImageFlechasIzquierdaDerecha->Picture->Bitmap);
      ImageFlechasIzquierdaDerecha->Refresh();
   }
}
// ---------------------------------------------------------------------------

void TFMain::estado(ES_Estado_t es) {
   switch(es) {
      case ES_Desconectado:
         LabelEstado->Caption = "Desconectado";
         LabelEstado->Color = clDkGray;
         LabelEstado->Font->Color = clWhite;
         break;
      case ES_NoDetectado:
         LabelEstado->Caption = "Sin Casco";
         LabelEstado->Color = clYellow;
         LabelEstado->Font->Color = clBlack;
         break;
      case ES_Error:
         LabelEstado->Caption = "Error";
         LabelEstado->Color = clRed;
         LabelEstado->Font->Color = clWhite;
         break;
      case ES_Conectado:
         LabelEstado->Caption = "Conectado";
         LabelEstado->Color = clGreen;
         LabelEstado->Font->Color = clWhite;
         break;
      case ES_Perfil:
         LabelEstado->Caption = "Sin Perfil";
         LabelEstado->Color = clPurple;
         LabelEstado->Font->Color = clWhite;
         break;
      case ES_Ruidoso:
         LabelEstado->Caption = "Seal Ruidosa";
         LabelEstado->Color = clYellow;
         LabelEstado->Font->Color = clBlack;
			break;
		case ES_NoSenal:
			LabelEstado->Caption = "No Enlazado";
			LabelEstado->Color = clYellow;
			LabelEstado->Font->Color = clBlack;
			break;
   default: ;
   }
}
// ---------------------------------------------------------------------------

void TFMain::senal(SN_Senal_t sn) {
   if (conectado) {
      switch(sn) {
         case SN_Mala:
            ProgressBarSenal->BarColor = clYellow;
            ProgressBarSenal->Position = 1;
            break;
         case SN_Buena:
            ProgressBarSenal->BarColor = clLime;
            ProgressBarSenal->Position = 2;
            break;
      default:
         ProgressBarSenal->Position = 0;
      }
   } else ProgressBarSenal->Position = 0;
}
// ---------------------------------------------------------------------------

void TFMain::bateria(int carga, int maxima) {
   if ((conectado) && (carga >= 0)) {
      maxima--; // Siempre excede en 1
      CGaugeBateria->BackColor = clWhite;
      CGaugeBateria->MaxValue = maxima;
      CGaugeBateria->Progress = carga;
      int mitad = maxima / 2;
      int cuarto = maxima / 4;
      // Colores para 1/2 y 1/4 de carga
      if (carga <= cuarto)
         CGaugeBateria->ForeColor = clRed;
      else if (carga <= mitad)
         CGaugeBateria->ForeColor = clYellow;
      else
         CGaugeBateria->ForeColor = clGreen;
   } else {
      // Desconectado
      CGaugeBateria->ForeColor = clBlack;
      CGaugeBateria->BackColor = clBtnFace;
      CGaugeBateria->MaxValue = 10;
      CGaugeBateria->Progress = 0;
   }
}
// ---------------------------------------------------------------------------

void TFMain::fuerza(unsigned short fu) {
   if (conectado) {
      CGaugeFuerza->BackColor = clYellow;
      CGaugeFuerza->ForeColor = clBlue;
      CGaugeFuerza->Progress = fu;
   } else {
      // Desconectado
      CGaugeFuerza->BackColor = clBtnFace;
      CGaugeFuerza->ForeColor = clBlack;
      CGaugeFuerza->Progress = 0;
   }
}
// ---------------------------------------------------------------------------

void TFMain::desviacion(short inc) {
   if (conectado) {
      CGaugeGiro->BackColor = clYellow;
      CGaugeGiro->ForeColor = clBlue;
      CGaugeGiro->Progress = inc;
   } else {
      // Desconectado
      CGaugeGiro->BackColor = clBtnFace;
      CGaugeGiro->ForeColor = clBlack;
      CGaugeGiro->Progress = 0;
   }
}
// ---------------------------------------------------------------------------

void TFMain::entrenamiento(UnicodeString fichero, float nivel) {
   LabelFicheroPerfil->Caption = ExtractFileName(fichero);
   LabelFicheroPerfil->Hint = fichero;
   // Pasa a porcentaje
   unsigned short valor = ceil(nivel*100);
   ProgressBarEntrenamiento->Position = valor;
   // Colores segn el nivel de entrenamiento
   if (valor < 33) ProgressBarEntrenamiento->BarColor = clRed;
   else if (valor < 67) ProgressBarEntrenamiento->BarColor = clYellow;
   else ProgressBarEntrenamiento->BarColor = clGreen;
   // Actualiza el Hint
   ProgressBarEntrenamiento->Hint = "Entrenamiento: " + FormatFloat("0.0%",nivel*100);
}
// ---------------------------------------------------------------------------

void TFMain::formulario() {
   // Activaciones de los controles
   ActionConectar->Enabled = !manual;
   ActionManual->Enabled = !conectado;
   ActionCasco->Enabled = !conectado;

   TrackBarUmbralAvRet->Enabled = conectado;
   TrackBarUmbralGiro->Enabled = conectado;
   TrackBarAvance->Enabled = conectado;
   TrackBarRetroceso->Enabled = conectado;
   ShapeIndicadorFuerza->Visible = conectado;
   ShapeIndicadorGiro->Visible = conectado;

   labelsTiempoConexion(conectado);
   labelsFicheroPerfil(conectado);

   // Entrenamiento del perfil
   SpeedButtonPerfil->Enabled = conectado;
   SpeedButtonCentrarCasco->Enabled = conectado;
   entrenamiento("",0);

   // Imagen casco desconectado
   imagenCasco(false,NULL);

   if (manual) {
      // Manual
      RadioGroupMotor->Enabled = false;
      imagen(AC_Ninguna, GI_Ninguno, false, false);
      senal(SN_Ninguna);
      bateria(0, 1);
      fuerza(0);
      desviacion(0);
      editsServidor(false);
      ActionDesconectar->Visible = false;
      ActionConectar->Visible = true;
      estado(ES_Desconectado);
      grafica->estado(false);
      grafica->limpiar();
      LabelTiempoConexion->Caption = "";
   } else if (conectado) {
      // Conectado
      editsServidor(false);
      RadioGroupMotor->Enabled = false;
      imagen(AC_Ninguna, GI_Ninguno, false, false);
      ActionDesconectar->Visible = true;
      ActionConectar->Visible = false;
      //estado(ES_Conectado);
      PaintBoxGrafica->Color = clWhite;
      grafica->estado(true);
   } else {
      // Desconectado
      RadioGroupMotor->Enabled = true;
      senal(SN_Ninguna);
      bateria(0, 1);
      fuerza(0);
      desviacion(0);
      imagen(AC_Ninguna, GI_Ninguno, false, false);
      estado(ES_Desconectado);
      ActionDesconectar->Visible = false;
      ActionConectar->Visible = true;
      RadioGroupMotorClick(NULL);
      estado(ES_Desconectado);
      PaintBoxGrafica->Color = clBtnFace;
      grafica->estado(false);
      grafica->limpiar();
   }
   Application->ProcessMessages();
}
// ---------------------------------------------------------------------------

void TFMain::editsServidor(bool activo)
{
   EditIp1->Enabled = activo;
   EditIp2->Enabled = EditIp1->Enabled;
   EditIp3->Enabled = EditIp1->Enabled;
   EditIp4->Enabled = EditIp1->Enabled;
   EditPuerto->Enabled = activo;
   // Colores
   if (EditIp1->Enabled) {
      EditIp1->Color = clWindow;
   } else {
      EditIp1->Color = clBtnFace;
   }
   EditIp2->Color = EditIp1->Color;
   EditIp3->Color = EditIp1->Color;
   EditIp4->Color = EditIp1->Color;

   if (EditPuerto->Enabled) {
      EditPuerto->Color = clWindow;
   }
   else {
      EditPuerto->Color = clBtnFace;
   }
}
// ---------------------------------------------------------------------------

void TFMain::labelsTiempoConexion(bool activo)
{
   LabelTiempoConexionEtq->Enabled = activo;
   LabelTiempoConexion->Enabled = activo;
   // Colores
   if (activo) LabelTiempoConexion->Color = clWindow;
   else LabelTiempoConexion->Color = clDkGray;
}
// ---------------------------------------------------------------------------

void TFMain::labelsFicheroPerfil(bool activo)
{
   LabelFicheroPerfil->Enabled = activo;
   // Colores
   if (activo) {
      if (perfilCargado) LabelFicheroPerfil->Color = clYellow;
      else LabelFicheroPerfil->Color = clRed;
   } else LabelFicheroPerfil->Color = clDkGray;

}
// ---------------------------------------------------------------------------

void TFMain::accionManual(AC_Accion_t ac, GI_Giro_t gi)
{
   // Ejecuta la accin manual
   actuador->movimiento(ac, gi);
   // Representa en la imagen
   imagen(ac, gi, true, true);
}
//---------------------------------------------------------------------------

void TFMain::imagenCasco(bool estado_casco, int *contactos)
{
   const TShape *electrodos[NUM_ELECTRODOS] = {ShapeElectodo01, ShapeElectodo02,
                                               ShapeElectodo03, ShapeElectodo04,
                                               ShapeElectodo05, ShapeElectodo06,
                                               ShapeElectodo07, ShapeElectodo08,
                                               ShapeElectodo09, ShapeElectodo10,
                                               ShapeElectodo11, ShapeElectodo12,
                                               ShapeElectodo13, ShapeElectodo14,
                                               ShapeElectodo15, ShapeElectodo16};
   unsigned short i;
   unsigned short nuevaImagen;

   if (conectado) {
      if (contactos != NULL) {
         // Buen funcionamiento
         nuevaImagen = 1;
         for (i = 0; i < NUM_ELECTRODOS; i++) {
            if (electrodos[i]->Brush->Color != ESTADOS[contactos[i]]) {
               electrodos[i]->Brush->Color = ESTADOS[contactos[i]];
            }
         }
      } else {
         // Seal baja
         nuevaImagen = 2;
         for (i = 0; i < NUM_ELECTRODOS; i++) {
            electrodos[i]->Brush->Color = clBlack;
         }
      }
   } else {
      // Desconectado
      nuevaImagen = 0;
      for (i = 0; i < NUM_ELECTRODOS; i++) {
         electrodos[i]->Brush->Color = clDkGray;
      }
   }
   // Refresca si ha cambiado la imagen de fondo
   if (nuevaImagen != imgCasco) {
      imgCasco = nuevaImagen;
      ImageListCasco->GetBitmap(imgCasco,ImageCasco->Picture->Bitmap);
      ImageCasco->Repaint();
   }
}
//---------------------------------------------------------------------------

void TFMain::tiempoConexion(float tiempo)
{
   // Formatea y vuelca el valor en la etiqueta
   LabelTiempoConexion->Caption = FormatFloat("0.000",tiempo);
}

//---------------------------------------------------------------------------

void __fastcall TFMain::ActionConectarExecute(TObject *Sender)
{
   UnicodeString cadenaOk, cadenaError, cadenaIp;

   // Inicializa variables
   continuar = true;
   perfilCargado = false;
   cargarPerfil = false;
   inicializarGiro = false;

   bool activoAccion = false;
   bool activoGiro = false;
   bool cambioGiro = false;
   bool emulador;

	AC_Accion_t ac, ac_mov;
	SN_Senal_t wifi;
	GI_Giro_t gi, gi_mov, giLeido, giImg;

   unsigned int potencia = 0;
   unsigned int potenciaGiro = 0;
   unsigned int retardoGiro = 0;
   unsigned short gr[NUM_GRAFICAS];

   int carga_bat = 0;
   int carga_max = 0;
   int electrodos[NUM_ELECTRODOS];
   bool electrodos_ok = false;

   // Conecta segn la seleccin
   if (RadioGroupMotor->ItemIndex == 0) {
      // Conexin con el motor
      con = new TConexionCasco();
      cadenaOk = "Conectado al Engine Emotiv";
      cadenaError = "Fallo iniciando el Engine Emotiv";
      emulador = false;
   } else {
      // Conexin con el emulador
      cadenaIp = EditIp1->Text+"."+EditIp2->Text+"."+EditIp3->Text+"."+EditIp4->Text;
		con = new TConexionEmulador(AnsiString(cadenaIp).c_str(), StrToInt(EditPuerto->Text));
      cadenaOk = "Conectado con EmoComposer en " + cadenaIp + ":" + EditPuerto->Text;
      cadenaError = "No se puede conectar con EmoComposer en " + cadenaIp + ":" + EditPuerto->Text;
      emulador = true;
   }

   if (con->conectar()) {
		registro(RG_Mensaje, cadenaOk);
      conectado = true;

      // Optimizacin
      if (con->inicializar()) {
         registro(RG_Mensaje, "Optimizacin activada");
         registro(RG_Mensaje, con->cadenaOptimizacion());
      }
      else {
         registro(RG_Error, "No se ha podido activar la optimizacin");
         registro(RG_Error, con->cadenaError());
      }
   } else {
      registro(RG_Error, cadenaError);
      registro(RG_Error, con->cadenaError());
   }

   // Estado grfico del formulario
   formulario();
   tiempoConexion(0.0);
   gi = GI_Ninguno;
	giImg = GI_Ninguno;

	// Bucle principal
	while (conectado && continuar) {

		if (con->eventoSiguiente()) { // Si hay un evento desde el casco

			tiempoConexion(con->tiempoConexion());

			if (con->estadoActualizado()) { // Nuevo estado Emotivo

            if (inicializarGiro) { // Si hay una peticin para inicializar el acelermetro
               inicializarGiro = false;
               PanelCalibrandoAcelerometros->Visible = true;
               Application->ProcessMessages();
               Sleep(1000);
               if (!con->inicializaGiro()) registro(RG_Error, con->cadenaError());
               Sleep(1000);
               PanelCalibrandoAcelerometros->Visible = false;
               Application->ProcessMessages();
            }

            // indicadores
            con->cargaBateria(&carga_bat, &carga_max);
            bateria(carga_bat, carga_max);
				wifi = con->nivelSenal();
            senal(wifi);

				if (perfilCargado || emulador) { // debe estar cargado el perfil
															// o bien estar conectado al emulador

					if (wifi == SN_Buena) {  // la seal del enlace debe ser buena

						// Imgenes
						con->estadoElectrodos(electrodos, &electrodos_ok);
						imagenCasco(electrodos_ok,electrodos);

						if (electrodos_ok) { // Si la deteccin es fiable

							// conectado y recibiendo
							estado(ES_Conectado);

							// Accin
							con->accion(&ac,&potencia);
							activoAccion = potencia >= (unsigned int) TrackBarUmbralAvRet->Position;
							fuerza(potencia);

							// Girscopo
							con->giro(&giLeido); // accin de giro
							potenciaGiro = con->aceleracionGiro(); // nivel de aceleracin de giro
							desviacion(potenciaGiro);

							retardoGiro++;

							if (retardoGiro > 10) {

								cambioGiro = cambioGiro || (giLeido == GI_Ninguno); // Si deja de detectar el mismo sentido
																									 // ya puede ser desactivado

								if (potenciaGiro >= (unsigned int) TrackBarUmbralGiro->Position) { // supera umbral

									if (activoGiro) { // si estaba girando
										if (cambioGiro) { // si hubo cambio
											if (giLeido == gi) { // si el leido es igual al actual
												gi = GI_Ninguno;  // se desasigna
												activoGiro = false;  // se desactiva
												retardoGiro = 0;
											}
										}
									} else { // no estaba girando
										if (giLeido != GI_Ninguno) { // si se ha leido una direccin de giro
											gi = giLeido;  // se asigna
											activoGiro = true;  // y se activa
											retardoGiro = 0;
										}
									}
								}
							}

							// Gestiona la imagen a mostrar
							if (gi == GI_Ninguno) giImg = giLeido;
							else giImg = gi;

							// Registra el estado leido
							if (ActionEstados->Checked) registro(RG_Estado, con->cadenaEstado());

						} else {
							ac = AC_Ninguna; // Deteccin no fiable
							estado(ES_Ruidoso);
						}
					} else {
						ac = AC_Ninguna; // Sin seal
						estado(ES_NoSenal);
					}

				} else {
					ac = AC_Ninguna;
               estado(ES_Perfil);
            }

         } else if (con->dispositivoConectado()) { // conexin dispositivo
            tiempoConexion(0.0);
            estado(ES_Conectado);
            registro(RG_Mensaje, "Dispositivo detectado");
         } else if (con->dispositivoDesconectado()) { // desconexin dispositivo
            tiempoConexion(0.0);
            estado(ES_NoDetectado);
            imagenCasco(false,NULL);
            registro(RG_Mensaje, "Dispositivo no detectado");
         } else registro(RG_Estado, con->tipoEstado());  // otro estado

         // Imagen accin
         imagen(ac, giImg, activoAccion, activoGiro);

         // Actuador
         if (activoAccion) ac_mov = ac;
         else ac_mov = AC_Ninguna;
         if (activoGiro) gi_mov = gi;
         else gi_mov = GI_Ninguno;
         actuador->movimiento(ac_mov, gi_mov);

         // Grfica
         for (unsigned int i = 0; i < NUM_GRAFICAS; i++) {
            gr[i] = 0;
         }
         switch (ac) {
            case AC_Ninguna:  gr[SG_Ninguna] = 10; break;
            case AC_Centro:   gr[SG_Centro] = 10; break;
            case AC_Adelante: gr[SG_Adelante] = potencia; break;
            case AC_Atras:    gr[SG_Atras] = potencia; break;
         default:
            ;
         }
         switch (giLeido) {
            case GI_Izquierda: gr[SG_Izquierda] = potenciaGiro; break;
            case GI_Derecha:   gr[SG_Derecha] = potenciaGiro; break;
         default:
            ;
         }
         grafica->anadir(gr);

      } // No hay eventos nuevos

      if (cargarPerfil) { // Peticin de carga de perfil
			cargarPerfil = false;
			if (con->cargaPerfil(AnsiString(ficheroPerfil).c_str())) {
				perfilCargado = true;
				ficheroPerfil = OpenDialog->FileName;
				registro(RG_Mensaje,con->cadenaPerfil());
				entrenamiento(OpenDialog->FileName,con->nivelEntrenamiento());
				TrackBarAvance->Position = con->getSensibilidadAvance();
				TrackBarAvanceChange(TrackBarAvance);
				TrackBarRetroceso->Position = con->getSensibilidadRetroceso();
				TrackBarRetrocesoChange(TrackBarRetroceso);
         } else {
            perfilCargado = false;
            ficheroPerfil = "";
            registro(RG_Error,"Error cargando el perfil desde " + OpenDialog->FileName);
            registro(RG_Error,con->cadenaError());
            entrenamiento("",0);
         }
         labelsFicheroPerfil(conectado);
         SpeedButtonPerfil->Enabled = true;
      }

      // Refresco cambios en la GUI
      Application->ProcessMessages();
   }

   actuador->movimiento(AC_Ninguna, GI_Ninguno);

   if (conectado) {

      // Desconectar
      if (con->desconectar()) registro(RG_Mensaje, "Desconectado");
      else {
         registro(RG_Error, "Error desconectando");
         registro(RG_Error, con->cadenaError());
      }
   }

   delete con;
   conectado = false;
   perfilCargado = false;

   // Actualiza el estado del formulario
   formulario();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::RadioGroupMotorClick(TObject *Sender)
{
   editsServidor((RadioGroupMotor->ItemIndex == 1));
}
// ---------------------------------------------------------------------------

void __fastcall TFMain::ActionManualExecute(TObject *Sender)
{
   manual = true;
   formulario();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionCascoExecute(TObject *Sender)
{
   manual = false;
   formulario();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionDesconectarExecute(TObject *Sender)
{
   continuar = false;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ImageFlechasArribaAbajoMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
   if (manual) {
      if ((X >= 6)  && (X <= 28) &&
          (Y >= 10) && (Y <= 60)) accionManual(AC_Adelante, giroManual); // arriba
      else if ((X >= 6)   && (X <= 28) &&
               (Y >= 100) && (Y <= 150)) accionManual(AC_Atras, giroManual); // abajo
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ImageFlechasIzquierdaDerechaMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
   if (manual) {
      if ((X >= 10) && (X <= 60) &&
          (Y >= 6)  && (Y <= 28) &&
          (giroManual != GI_Izquierda)) giroManual = GI_Izquierda;  // izquierda
      else if ((X >= 100) && (X <= 150) &&
               (Y >= 6)  && (Y <= 28) &&
               (giroManual != GI_Derecha)) giroManual = GI_Derecha; // derecha
      else giroManual = GI_Ninguno;
      accionManual(AC_Ninguna, giroManual);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ImageFlechasArribaAbajoMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
   accionManual(AC_Ninguna, giroManual);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionActuadorExecute(TObject *Sender)
{
   bool activado = actuador->habilitar(ActionActuador->Checked);
   if (activado) registro(RG_Mensaje,"Actuador Habilitado");
   else {
      registro(RG_Mensaje,"Actuador Inhabilitado");
      if (ActionActuador->Checked) { // No se activ cuando deba
         ActionActuador->Checked = false;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::FormDestroy(TObject *Sender)
{
   delete actuador;
   delete grafica;

   // Guarda la configuracin
   TIniFile *F;
   try {
      F = new TIniFile(ficheroINI);
      F->WriteInteger("UMBRALES","AVRET",TrackBarUmbralAvRet->Position);
      F->WriteInteger("UMBRALES","GIRO",TrackBarUmbralGiro->Position);
   } __finally {
      delete F;
   }

}
//---------------------------------------------------------------------------

void __fastcall TFMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   CanClose = !conectado;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::EditPuertoExit(TObject *Sender)
{
   int num;
   if ((!TryStrToInt(EditPuerto->Text,num)) ||
       (num < 1) ||
       (num > 65535))
       EditPuerto->Text = PUERTO_EMULADOR; // Si no es vlido pone el valor por defecto
}
//---------------------------------------------------------------------------

void __fastcall TFMain::EditIp1Exit(TObject *Sender)
{
   int num;
   TEdit *edit = dynamic_cast<TEdit *>(Sender);
   if ((!TryStrToInt(edit->Text,num)) ||
       (num < 0) ||
       (num > 255))
       edit->Text = "0"; // Si no es vlido pone 0
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionCargaPerfilExecute(TObject *Sender)
{
   if (OpenDialog->Execute()) {
      cargarPerfil = true; // Peticin de carga de perfil
      ficheroPerfil = OpenDialog->FileName;
      SpeedButtonPerfil->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::TrackBarUmbralAvRetChange(TObject *Sender)
{
   TrackBarUmbralAvRet->Hint = "Umbral Avance/Retroceso: " + FormatFloat("0%",TrackBarUmbralAvRet->Position * 10);
   ShapeIndicadorFuerza->Top = CGaugeFuerza->Top + CGaugeFuerza->Height - 1  -
                               TrackBarUmbralAvRet->Position * CGaugeFuerza->Height / 10;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::TrackBarUmbralGiroChange(TObject *Sender)
{
   TrackBarUmbralGiro->Hint = "Umbral Giro: " + FormatFloat("0%",TrackBarUmbralGiro->Position * 10);
   ShapeIndicadorGiro->Left = CGaugeGiro->Left - 1 +
                              TrackBarUmbralGiro->Position * CGaugeGiro->Width / 10;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::TrackBarAvanceChange(TObject *Sender)
{
   TrackBarAvance->Hint = "Sensibilidad Avance: " + FormatFloat("0",TrackBarAvance->Position);
   if (!con->setSensibilidadAvance(TrackBarAvance->Position)) {
      registro(RG_Error,con->cadenaError());
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::TrackBarRetrocesoChange(TObject *Sender)
{
   TrackBarRetroceso->Hint = "Sensibilidad Retroceso: " + FormatFloat("0",TrackBarRetroceso->Position);
   if (!con->setSensibilidadRetroceso(TrackBarRetroceso->Position)) {
      registro(RG_Error,con->cadenaError());
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionLimpiarExecute(TObject *Sender)
{
   RichEditRegistro->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionEstadosExecute(TObject *Sender)
{
   ActionEstados->Checked = !ActionEstados->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionGuardarExecute(TObject *Sender)
{
   if (SaveDialog->Execute()) { // Guarda el texto del registro en disco
      unsigned long escritos;
      HANDLE h;
		h = CreateFile(SaveDialog->FileName.w_str(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
      if (h == INVALID_HANDLE_VALUE) {
         MessageDlg("Se ha producido un error abriendo el fichero\n"+GetLastError(), mtError, TMsgDlgButtons()<<mbOK, NULL);
      } else {
         if (!WriteFile(h, AnsiString(RichEditRegistro->Lines->Text).c_str(), (RichEditRegistro->Lines->Text).Length(), &escritos, NULL)) {
            MessageDlg("Se ha producido un error escribiendo en el fichero\n"+GetLastError(), mtError, TMsgDlgButtons()<<mbOK, NULL);
         }
         if (!CloseHandle(h)) {
            MessageDlg("Se ha producido un error cerrando el fichero\n"+GetLastError(), mtError, TMsgDlgButtons()<<mbOK, NULL);
         } else {
            MessageDlg("El fichero se ha guardado con xito", mtCustom, TMsgDlgButtons()<<mbOK, NULL);
         }
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionAyudaExecute(TObject *Sender)
{
   Application->HelpCommand(HELP_CONTEXT, 0);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionAcercaDeExecute(TObject *Sender)
{
   // Visualiza la ventana de Acerca de...
   Application->CreateForm(__classid(TFAboutBox), &FAboutBox);
   FAboutBox->datos(NOMBRE,PROYECTO,FECHA_VERSION + " - " + AUTOR,"Versin: " + VERSION + " (" + BUILD + ")");
   FAboutBox->ShowModal();
   FAboutBox->Release();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::ActionCentrarCascoExecute(TObject *Sender)
{
   inicializarGiro = true; // Solicitud de centrar el acelermetro
}
//---------------------------------------------------------------------------



